#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Crée le Mon Feb 14 10:34:29 2022
@auteur: david ALBERTO
(www.astrolabe-science.fr)
Ce code utilise le module mendeleev (données sur les éléments chimiques)
Dans ce module, un élément est appelé par element(N) pour récupérer
les données suiantes :
    - symbole
    - group_id : c'est le numéro de colonne
    - period : c'est le numéro de ligne
    - atomic_weight : la masse atomique (en u), ici assimilée à la masse molaire atomique (en g/mol)
"""

 # Paramètres personnalisables-----
blocs=True#True : cases colorées selon les blocs. False : incolores
masses_molaires=True#affichage ou pas
"""
Les paramètres suivants sont liés entre eux, pour le rendu final.
Ils doivent être ajustés manuellement :
    - dimensions de la figure
    - valeurs des paramètres d'échelle a et b
    - taille des caractères
    - les différents offsets en x et en y
"""
#facteurs d'échelle :
a=0.22 #en abscisses
b=0.28 #en ordonnées. NB : les ordonnées sont négatives

# Réglages d'écriture du symbole des éléments :
taille_symbole=16
c_symbole='k'#couleur symbole

# Réglages d'écriture du numéro atomique :
x_offset_Z=-0.32
y_offset_Z=0.33
taille_Z=9
c_Z='k'#couleur numéro atomique

# Réglages d'écriture des terres rares :
y_offset_La=2.5#décalage vertical lanthanides
y_offset_Ac=y_offset_La#décalage vertical actinides

# Réglages d'écriture du nom des éléments :
x_offset_nom=-0.0
y_offset_nom=-0.4
c_nom='blue'
taille_nom=7

# Réglages d'écriture de la masse molaire des éléments :
x_offset_masse=0.45
y_offset_masse=0.33
c_masse='red'
taille_masse=9

# Couleurs des blocs :
c_bloc_s='orange'
c_bloc_p='lightsteelblue'
c_bloc_d='lightyellow'
c_bloc_f='yellowgreen'


# ---------------------------------
# Appel des modules
import matplotlib.pyplot as plt
from mendeleev import element
#---------------------------------------------------------

def chiffSignif(n):#renvoie un nombre n avec 3 chiffres significatifs
    if n<10:
        valeur='{:.2f}'.format(n)
    elif n<100:
        valeur= '{:.1f}'.format(n)
    else:
        valeur= '{:.0f}'.format(n)
    return valeur.replace('.',',')#virgule à la place du point

#---------------------------------------------------------
noms=['Hydrogène','Hélium','Lithium','Béryllium','Bore','Carbone','Azote','Oxygène',
      'Fluor','Néon','Sodium','Magnésium','Aluminium','Silicium','Phosphore','Soufre',
      'Chlore','Argon','Potassium','Calcium','Scandium','Titane','Vanadium','Chrome',
      'Manganèse','Fer','Cobalt','Nickel','Cuivre','Zinc','Gallium','Germanium',
      'Arsenic','Sélénium','Brome','Krypton','Rubidium','Strontium','Yttrium',
      'Zirconium','Niobium','Molybdène','Technétium','Ruthénium','Rhodium',
      'Palladium','Argent','Cadmium','Indium',
'Étain','Antimoine','Tellure','Iode','Xénon','Césium','Baryum','Lanthane','Cérium',
'Praséodyme','Néodyme','Prométhium','Samarium','Europium','Gadolinium','Terbium',
'Dysprosium','Holmium','Erbium','Thulium','Ytterbium','Lutetium','Hafnium','Tantale',
'Tungstène','Rhénium','Osmium','Iridium','Platine','Or','Mercure','Thallium','Plomb',
'Bismuth','Polonium','Astate','Radon','Francium','Radium','Actinium','Thorium','Protactinium',
'Uranium','Neptunium','Plutonium','Americium','Curium','Berkelium','Californium','Einsteinium',
'Fermium','Mendelevium','Nobelium','Lawrencium','Rutherfordium','Dubnium','Seaborgium','Bohrium',
'Hassium','Meitnerium','Darmstadtium','Roentgenium','Copernicium','Nihonium','Flerovium',
'Moscovium','Livermorium','Tennessine','Oganesson',
]#le module mendeleev n'a que les noms des éléments en anglais
# ---------------------------------


#création des listes de coordonnées des éléments
"""
La ligne est la valeur de la période de l'élément.
La colonne est la valeur du paramètre "group_id" de l'élément.
Pour les terres rares, le module ne fournit pas de valeur pour "group_id" ; 
je me base donc sur la valeur de N pour établir un numéro de colonne.
Les listes "ligne" et "colonne" serviront ensuite pour toutes les indications
(masse molaire, N° atomique, symbole, nom).
"""
ligne=[]
colonne=[]

for N in range(1,118+1):
    if N>=58 and N<=71:#lanthanides
        colonne.append((3+N-57)*a)
        ligne.append((element(N).period+y_offset_La)*b)
    elif N>=90 and N<=103:#actinides
        colonne.append((3+N-89)*a)
        ligne.append((element(N).period+y_offset_Ac)*b)
    else:
        colonne.append((element(N).group_id)*a)
        ligne.append((element(N).period)*b)
# ---------------------------------
# paramètres du graphique
fig=plt.figure(figsize=(29.7/2.54,21/2.54),tight_layout=True,facecolor='slateblue')

plt.rcParams["font.family"] = "Serif"
# plt.rcParams["font.family"] = "Liberation Serif"# ou autre police installée (police Serif conseillée)

plt.rcParams.update({"text.usetex": True}) #utiliser la police LaTeX (optionnel)
ax=plt.subplot(111)

plt.title("Classification périodique des éléments chimiques",fontsize=20,y=0.92)
plt.xlim(0,19*a)
plt.ylim(-10.5*b,0)
plt.xticks([])
plt.yticks([])
ax.set_aspect('equal')
# ---------------------------------
# Remplissage du tableau :
for N in range(1,118+1):
    elem=element(N)
    plt.text(colonne[N-1],-ligne[N-1],elem.symbol,fontsize=taille_symbole,va='center',ha='center')#symbole
    if blocs==True:
        if elem.block=='s':
            couleur_case=c_bloc_s
        elif elem.block=='p':
            couleur_case=c_bloc_p
        elif elem.block=='d':
            couleur_case=c_bloc_d
        else : couleur_case=c_bloc_f
    else: couleur_case='none'
    rectangle=plt.Rectangle((colonne[N-1]-0.5*a,-(ligne[N-1]+0.5*b)),a,b,facecolor=couleur_case,edgecolor='k')#case
    ax.add_patch(rectangle)
    if N in [91,101,104,110,116]:#éléments aux noms longs
        tailleDuNom=6
    else:tailleDuNom=taille_nom
    plt.text(colonne[N-1]+x_offset_Z*a,-ligne[N-1]+y_offset_Z*b,N,c=c_Z,fontsize=taille_Z,ha='center')#numéro atomique
    plt.text(colonne[N-1]+x_offset_nom*a,-ligne[N-1]+y_offset_nom*b,r'\textsf{%s}'%(noms[N-1]),c=c_nom,fontsize=tailleDuNom,ha='center')#nom
    # autre option, sans le code LaTeX :
    # plt.text(colonne[N-1]+x_offset_nom*a,-ligne[N-1]+y_offset_nom*b,noms[N-1],c=c_nom,fontsize=tailleDuNom,ha='center')#nom
    if masses_molaires==True:
        plt.text(colonne[N-1]+x_offset_masse*a,-ligne[N-1]+y_offset_masse*b,chiffSignif(elem.atomic_weight),c=c_masse,fontsize=taille_masse,ha='right')#masse
    tailleDuNom=taille_nom

# Légende :
x=5*a
y=-b*4
# Les positions exactes qui suivent sont à ajuster manuellement !
plt.annotate('numéro atomique Z', (x-0.065,y+y_offset_Z-0.7*b),xytext=(x-0.285,y+y_offset_Z+0.1*b),c=c_Z,arrowprops=dict(arrowstyle='-',color=c_Z))
plt.annotate(r'masse molaire ($g.mol^{-1}$)', (x+0.035+2*a,y+y_offset_masse-0.7*b),xytext=(x+0.22,y+y_offset_masse+0.1*b),c=c_masse,fontsize=taille_masse,arrowprops=dict(arrowstyle='-',color=c_masse))

# Trait pour les terres rares :
plt.plot([3.5*a,3.5*a],[-7.5*b,-8.0*b],'--',c='gray',lw=1)

NomFichier="TableauPeriodique"
fig.savefig(NomFichier+'.png',dpi=400)
fig.savefig(NomFichier+'.pdf')